"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Inject = void 0;
const startup_1 = require("../startup");
/**
 * Injects instance into the constructor argument or into the class property.
 * In parameter inject case all providers have been instantiated during its class instantiation.
 * In property inject case all providers have been instantiated 'by demand'.
 * @param type Provider type.
 */
function Inject(type) {
    return (target, key, index) => {
        if (type === undefined) {
            throw new Error(`Dependency injector: Type is undefined or circular dependency detected in type [${target.name}]`);
        }
        if (index !== undefined) {
            startup_1.DependencyRegistry.addDependency(target, type, index);
            return;
        }
        const k = Symbol(key.toString());
        Object.defineProperty(target, k, {
            value: undefined,
            enumerable: false,
            writable: true
        });
        Object.defineProperty(target, key, {
            get: function () {
                let instance = this[k];
                if (!instance) {
                    instance = startup_1.DependencyResolver.resolve(type);
                    this[k] = instance;
                }
                return instance;
            },
            enumerable: true
        });
    };
}
exports.Inject = Inject;
//# sourceMappingURL=inject.js.map