"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.DependencyResolver = void 0;
const injection_scope_enum_1 = require("./types/injection-scope.enum");
const literals_1 = require("./literals");
const dependency_not_registered_exception_1 = require("./types/dependency-not-registered-exception");
class DependencyResolver {
    constructor(registry, providers, instances) {
        this.registry = registry;
        this.providers = providers;
        this.instances = instances;
    }
    resolve(type) {
        const providedType = (this.providers.getProvider(type) || type);
        const info = this.registry.getTypeInfo(providedType);
        if (!info) {
            const ctor = this.getOriginalConstructor(providedType);
            throw new dependency_not_registered_exception_1.DependencyNotRegisteredException(ctor.name);
        }
        switch (info.scope) {
            case injection_scope_enum_1.InjectionScopeEnum.Transient: return this.instantiate(providedType, info.dependencies);
            default: return this.createAndGetSingletone(providedType, info.dependencies);
        }
    }
    createAndGetSingletone(type, deps) {
        let instance = this.instances.getInstance(type);
        if (!instance) {
            instance = this.instantiate(type, deps);
            this.instances.addInstance(type, instance);
        }
        return instance;
    }
    instantiate(type, dependencies) {
        if (!dependencies.length) {
            return new type();
        }
        const args = Array.from(dependencies).sort((left, right) => left.index - right.index).map(m => this.resolve(m.type));
        return new type(...args);
    }
    getOriginalConstructor(type) {
        if (!Object.getOwnPropertySymbols(type).includes(literals_1.Literals.WrapperObject)) {
            return type.constructor;
        }
        return this.getOriginalConstructor(type.prototype);
    }
}
exports.DependencyResolver = DependencyResolver;
//# sourceMappingURL=dependency-resolver.js.map