"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Registry = void 0;
const default_injection_options_1 = require("./types/default-injection-options");
class Registry {
    constructor() {
        this.registry = new Map();
    }
    register(type, options) {
        if (!this.registry.has(type)) {
            this.registry.set(type, {
                dependencies: [],
                scope: this.applyOptions(options).scope
            });
        }
    }
    addDependency(type, dep, index) {
        const info = this.registry.get(type) || { dependencies: [], scope: this.applyOptions({}).scope };
        const d = info.dependencies.find(f => f.type === dep);
        if (d && d.index === undefined) {
            d.index = index;
        }
        else {
            info.dependencies.push({ type: dep, index });
        }
        this.registry.set(type, info);
    }
    getTypeInfo(target) {
        return this.registry.get(target);
    }
    has(t) {
        return this.registry.has(t);
    }
    applyOptions(opt) {
        return Object.assign(new default_injection_options_1.DefaultInjectionOptions(), opt);
    }
}
exports.Registry = Registry;
//# sourceMappingURL=registry.js.map